@@ -8,11 +8,13 @@ This module contains the step functions that are called by SimBot.
 
 Almost all the work is done in `process_orcaflex`, the basic steps are:
 
-1. Load the model data - this will only return once it has been able to get a licence or has tried multiple times to get
-    one as defined in model.OrcaFlexJobOptions
+1. Load the model data - this will only return once it has been able to get a licence or
+    has tried multiple times to get one as defined in model.OrcaFlexJobOptions
 2. Define progress handlers if required
-3. Calculate the estimated size of the simulation and based on available RAM disable in-memory logging if required.
-4. Attempt statics, use the `static_search`_ method if defined in model tags. If statics fails the step function returns
+3. Calculate the estimated size of the simulation and based on available RAM disable
+    in-memory logging if required.
+4. Attempt statics, use the `static_search`_ method if defined in model tags. If statics
+    fails the step function returns
 5. Attempt dynamics. If dynamics fails the step function returns.
 6. Save the simulation file to qalx if required.
 7. Extract results if required
@@ -23,6 +25,8 @@ Almost all the work is done in `process_orcaflex`, the basic steps are:
 
 
 """
+import queue
+import threading
 import time
 import traceback
 from io import BytesIO
@@ -104,11 +108,22 @@ def process_orcaflex(job: QalxJob) -> None:
         job.entity["meta"]["state"] = OrcaFlexJobState(**state_dict).__dict__
         job.save_entity()
 
-    def update_job_progress(progress_dict: Mapping) -> None:
+    update_queue = queue.Queue()
+
+    def update_job_progress() -> None:
         """update the progress and save the item"""
+        time_of_last_update = time.time()
         if ofx.progress:
-            ofx.progress["data"] = progress_dict
-            job.session.item.save(ofx.progress)
+            while True:
+                progress = update_queue.get()
+                time_since_last_update = time.time() - time_of_last_update
+                update_queue.task_done()
+                if time_since_last_update > job.options.update_interval:
+                    ofx.progress["data"] = progress
+                    job.session.item.save(ofx.progress)
+                    time_of_last_update = time.time()
+
+    threading.Thread(target=update_job_progress, daemon=True).start()
 
     def update_job_warnings(warnings: Sequence[str]) -> None:
         """add all the warning texts, save the item"""
@@ -152,7 +167,7 @@ def process_orcaflex(job: QalxJob) -> None:
             percent=pc,
             pretty_time=pretty_time,
         )
-        update_job_progress(dp)
+        update_queue.put(dp)
         if (
             job.options.killable
         ):  # allows killing jobs/bots although this is not tested or
@@ -175,7 +190,7 @@ def process_orcaflex(job: QalxJob) -> None:
             time_to_go=model.simulationTimeToGo,
             pretty_time=nice_time(model.simulationTimeToGo),
         )
-        update_job_progress(sp)
+        update_queue.put(sp)
 
     def report_percent(model: OrcFxAPI.Model, percent: float):
         pp = dict(
@@ -183,7 +198,7 @@ def process_orcaflex(job: QalxJob) -> None:
             time_to_go=model.simulationTimeToGo,
             pretty_time=nice_time(model.simulationTimeToGo),
         )
-        update_job_progress(pp)
+        update_queue.put(pp)
 
     def report_error(error: str):
         update_job_state({"state": "Error", "error": error})
@@ -478,6 +493,7 @@ def process_orcaflex(job: QalxJob) -> None:
     # this allows `postprocessing_function` to access model data
     job.context["model"] = model
     job.entity["meta"]["processing_complete"] = True  # we done.
+    # update_queue.join() # wait until all the progress updates are pushed
 
 
 def post_process_orcaflex(job: QalxJob):
